"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scaleBetween = exports.createQueryDisplayLabel = exports.createQueryLabel = exports.createQueryHistoryAutocompleter = void 0;
const compass_components_1 = require("@mongodb-js/compass-components");
const mongodb_query_parser_1 = require("mongodb-query-parser");
const editor_1 = require("../editor");
const highlight_1 = require("@lezer/highlight");
const editor_2 = require("../editor");
function simplifyQueryStringForAutocomplete(queryString) {
    return queryString.replace(/[\s'"\n\t{}},]/g, '').toLowerCase();
}
function getMatchingQueryHistoryItemsForInput({ savedQueries, queryProperty, input: _input, }) {
    const input = simplifyQueryStringForAutocomplete(_input);
    if (input.length === 0) {
        return savedQueries;
    }
    return savedQueries.filter((query) => {
        const queryValue = query.queryProperties[queryProperty];
        if (typeof queryValue !== 'object') {
            const queryValueString = (0, mongodb_query_parser_1.toJSString)(queryValue) || '';
            if (!queryValueString) {
                return false;
            }
            const queryValueSimplified = simplifyQueryStringForAutocomplete(queryValueString);
            return queryValueSimplified.startsWith(input);
        }
        let inputToMatch = input;
        for (const [key, value] of Object.entries(queryValue).slice(0, 30)) {
            const fieldString = (0, mongodb_query_parser_1.toJSString)({ [key]: value }) || '';
            const fieldStringSimplified = simplifyQueryStringForAutocomplete(fieldString);
            if (fieldStringSimplified.startsWith(input)) {
                return true;
            }
            const inputIndex = inputToMatch.indexOf(fieldStringSimplified);
            if (inputIndex !== -1) {
                inputToMatch = inputToMatch.replace(fieldStringSimplified, '');
            }
        }
        if (inputToMatch.length === 0) {
            return true;
        }
        return false;
    });
}
const createQueryHistoryAutocompleter = ({ savedQueries, onApply, queryProperty, theme, }) => {
    return function queryCompletions(context) {
        if (savedQueries.length === 0) {
            return null;
        }
        const maxTime = savedQueries[savedQueries.length - 1].lastExecuted.getTime();
        const minTime = savedQueries[0].lastExecuted.getTime();
        const contextValue = context.state.sliceDoc(0, context.pos);
        const matchedQueries = getMatchingQueryHistoryItemsForInput({
            savedQueries,
            queryProperty,
            input: contextValue,
        });
        const options = matchedQueries.map((query) => ({
            displayLabel: createQueryDisplayLabel(query),
            label: createQueryLabel(query, queryProperty),
            type: query.type === 'recent' ? 'query-history' : 'favorite',
            detail: (0, compass_components_1.formatDate)(query.lastExecuted.getTime()),
            info: () => createInfo(query, theme).dom,
            apply: () => {
                onApply(query.queryProperties);
            },
            boost: scaleBetween(query.lastExecuted.getTime(), -99, 99, minTime, maxTime),
        }));
        return {
            from: context.pos,
            options: options,
        };
    };
};
exports.createQueryHistoryAutocompleter = createQueryHistoryAutocompleter;
const queryLabelStyles = (0, compass_components_1.css)({
    textTransform: 'capitalize',
    fontWeight: 'bold',
    fontFamily: compass_components_1.fontFamilies.default,
});
const queryCodeStyles = (0, compass_components_1.css)({
    fontFamily: compass_components_1.fontFamilies.code,
    margin: `${compass_components_1.spacing[50]}px`,
    marginLeft: `${compass_components_1.spacing[100]}px`,
    padding: 0,
    whiteSpace: 'pre-wrap',
});
function createQueryLabel(query, propertyName) {
    if (!query.queryProperties[propertyName]) {
        return '';
    }
    return (0, mongodb_query_parser_1.toJSString)(query.queryProperties[propertyName]) || '';
}
exports.createQueryLabel = createQueryLabel;
function createQueryDisplayLabel(query) {
    return Object.entries(query.queryProperties)
        .map(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value, 1);
        if (!formattedQuery)
            return '';
        const fieldKey = key === 'filter' ? '' : `${key}: `;
        return `${fieldKey}${formattedQuery}`;
    })
        .filter(Boolean)
        .reduce((acc, curr) => (acc ? `${acc}, ${curr}` : curr), '');
}
exports.createQueryDisplayLabel = createQueryDisplayLabel;
const javascriptExpressionLanguageParser = editor_1.languages['javascript-expression']().language.parser;
function createInfo(query, theme) {
    const customHighlighter = editor_2.highlightStyles[theme];
    const container = document.createElement('div');
    Object.entries(query.queryProperties).forEach(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value);
        const codeDiv = document.createElement('div');
        const label = document.createElement('label');
        label.className = queryLabelStyles;
        label.textContent = key;
        const code = document.createElement('pre');
        code.className = queryCodeStyles;
        function emit(text, classes) {
            const node = document.createTextNode(text);
            if (classes) {
                const span = document.createElement('span');
                span.appendChild(node);
                span.className = classes;
                code.appendChild(span);
            }
            else {
                code.appendChild(node);
            }
        }
        function emitBreak() {
            code.appendChild(document.createTextNode('\n'));
        }
        if (formattedQuery) {
            (0, highlight_1.highlightCode)(formattedQuery, javascriptExpressionLanguageParser.parse(formattedQuery), customHighlighter, emit, emitBreak);
        }
        codeDiv.append(label);
        codeDiv.appendChild(code);
        container.appendChild(codeDiv);
    });
    return {
        dom: container,
        destroy: () => {
            while (container.firstChild) {
                container.removeChild(container.firstChild);
            }
        },
    };
}
function scaleBetween(unscaledNum, newScaleMin, newScaleMax, originalScaleMin, originalScaleMax) {
    if (originalScaleMax === originalScaleMin)
        return newScaleMin + (newScaleMax - newScaleMin) / 2;
    return (((newScaleMax - newScaleMin) * (unscaledNum - originalScaleMin)) /
        (originalScaleMax - originalScaleMin) +
        newScaleMin);
}
exports.scaleBetween = scaleBetween;
//# sourceMappingURL=query-history-autocompleter.js.map